# -*- coding: utf-8 -*-
import argparse
from datetime import datetime as dt


def do(args: argparse.Namespace) -> None:
    """Add some tasks to your list

    `do` supports an unlimited number of tasks, but requires that tasks of more than 1
    word in length be enclosed in quotes. Single or double are fine - use whichever!
    To keep track of how long tasks have been on the list,  a timestamp of the form
    %y-%m-%d %H:%M is also added.

    Notes
    -----
        All tasks added at the same time will be added at the same rank. If you need to
        add multiple tasks at different ranks, you must call `to do` multiple times.

    Parameters
    ----------
    args : argparse.Namespace
        Arguments forwarded from the CLI. For this subcommand, this includes:
    args.file : Filer
        The TODO file to add to. From the root `to` command
    args.rank : int
        The importance to assign the new tasks.
    args.sort : Literal["rank", "date", "both", "none"]
        How to sort new tasks added to the list. From the root `to` command
    args.tasks : List[str]
        The task(s) to add to your list

    Returns
    -------
    None
        However, a confirmation message will be echoed to the terminal

    Examples
    --------
    $ to -s rank do 2 "An example task" "I'm very busy"

    """
    date = dt.now().strftime("%Y-%m-%d %H:%M")
    args.file.append([["", str(args.rank), date, item] for item in args.tasks])
    if args.sort != "none":
        keys = {"rank": [1], "date": [2], "both": [1, 2]}
        args.file.sort(keys[args.sort], header=True)
    ids = [str(x) for x in range(1, args.file.length)]
    ids.insert(0, "ID")
    args.file.write_col(ids, 0)
    print(f"{len(args.tasks)} task(s) added!")



import argparse
import os

from helpers.external_command import external_command


# file, sort, number, reminder, edit
def doing(args: argparse.Namespace) -> None:
    """See tasks in your list

    Notes
    -----
        `--edit` opens whatever editor is specified by your `EDITOR` env var.
        If one is not set, it will default to Vim. 
    
        Currently, `--reminder` has a dependency on `notify-send`. If this
        command is absent from your system, it will failt

    Parameters
    ----------
    args : argparse.Namespace
        Args passed from argparse. For this subcommand, these include:
    args.file : Filer
        The TODO file to echo. Derived from the root `to` command
    args.sort : Literal['both', 'none', rank', 'date']
        How to sort echoed tasks. Derived from the root `to` command
    args.number : int
        How many tasks to return
    args.reminder : bool
        Whether to use notify-send to create a pop-up
    args.edit : bool
        Whether to laucn an editor with your TODO file

    Returns
    -------
    None

    Example
    -------
    $ to doing -n 3

    """
    keys = {"rank": [1], "date": [2], "both": [1, 2]}
    if args.sort != "none":
        args.file.sort(keys[args.sort], header=True)
        ids = [str(x) for x in range(1, args.file.length)]
        ids.insert(0, "ID")
        args.file.write_col(ids, 0)

    lines = args.file.read()[: args.number + 1]

    if args.edit:
        # print(args.file.path)
        __editor__ = os.environ.get("EDITOR", "vim")
        external_command([__editor__, args.file.path])
    elif args.reminder:
        external_command(
            [
                "notify-send",
                "My TODOs",
                "\n".join(["\t".join(l) for l in lines]),
                "-u",
                "low",
                "-t",
                "5000",
            ]
        )
    else:
        for l in lines:
            print("\t".join(l))



import csv
import os
import shutil
from operator import itemgetter
from pathlib import Path
from typing import List

from helpers.external_command import external_command
from helpers.itemsetter import itemsetter


class Filer:

    """A class for gracefully handling file interactions with delimited data

    Designed particularly for passing context in a CLI, it is a thin wrapper for many
    common file I/O actions, including reading, writing (both lines and columns), and
    deleting.

    """

    def __init__(self, path: Path, create: bool = True, delimiter: str = "\t") -> None:
        """Initialise the Filer object
        
        Note
        ----
            The attribute self.length is created to hold the number of lines in the file.
            This is particularly useful for checking to see if a file has been altered.

        Parameters
        ----------
        path : Path
            A Posix filepath to the desired file/location
        create : bool
            If the file does not exist, should it be created?
        delimiter : str
            What delimited should be used with the file?


        Examples
        --------
        >>> from pathlib import Path
        >>> example = Filer(Path.home() / 'tmp.tsv')

        """
        self.path = Path(path)
        self.delimiter = delimiter
        if not os.path.isfile(self.path):
            if create:
                self.path.touch()
                self.length = 0
                # Specifically for initiating todos
                self.append([["ID", "Rank", "Date", "Task"]])
            else:
                raise OSError("File does not exist")
        self.length = int(
            external_command(["wc", "-l", self.path]).stdout.strip().split()[0]
        )

    def append(self, rows: List[List[str]]) -> None:
        """Appends contents of `rows` to self.path

        Note
        ----
            This will not over-write the contents of the file, mirroring the modes of
            `open()`_

        .. _open(): 
            https://docs.python.org/3.7/library/functions.html#open

        Parameters
        ----------
        rows : List[List[str]]
            A list of strings to write to self.path.

        Returns
        -------
        None

        Examples
        --------
        >>> example.append([['f','g', 'h'], ['i', 'j', 'k']])

        """
        with open(self.path, "a", newline="") as file:
            writer = csv.writer(file, delimiter=self.delimiter)
            writer.writerows(rows)
        self.length += len(rows)

    def delete(self, contains: str) -> bool:
        """Deletes all lines from self where `contains in line`

        Parameters
        ----------
        contains : str
            String to match for line deletion

        Returns
        -------
        bool
            True if successulf, false otherwise

        Example
        -------
        >>> example.delete('j')
        True

        """
        with open(self.path, "r") as r_file:
            reader = csv.reader(r_file, delimiter=self.delimiter)
            with open("tmp", "a") as w_file:
                writer = csv.writer(w_file, delimiter=self.delimiter)
                for line in reader:
                    if contains not in line:
                        writer.writerow(line)

        # If deleted, copy and return true
        tmp_length = int(
            external_command(["wc", "-l", Path("tmp")]).stdout.strip().split()[0]
        )
        if self.length != tmp_length:
            shutil.move("tmp", self.path)
            self.length = tmp_length
            return True

        # Otherwise, clean tmp and return false
        os.remove(Path("tmp"))
        return False

    def read(self) -> List[List[str]]:
        """Read the lines of self.path

        Note
        ----
            Reads in all lines, so will suffer on large files

        Parameters
        ----------
        None
    
        Returns
        -------
        List[List[str]]
            A list of lines where each line is a list of column values

        Examples
        --------
        >>> example.read()
        [['ID', 'Rank', 'Date', 'Task'], ['f', 'g', 'h']]

        """
        with open(self.path, "r", newline="") as file:
            reader = csv.reader(file, delimiter=self.delimiter)
            lines = []
            for line in reader:
                lines.append(line)
        return lines

    def sort(self, cols: List[int], header: bool = False) -> None:
        """Sort the contents of self.path by columns

        Parameters
        ----------
        cols : List[int]
            List of column indices indicating what to sort by.
            Remember, Python is 0-indexed   
        header : bool
            Whether or not row 0 is a header. If True, row 0 is skipped for sorting

        Returns
        -------
        None

        Example
        -------
        >>> example.sort([1, 2], header=False)

        """
        lines = self.read()
        if header:
            # Slice off header
            head = lines[0]
            entries = lines[1:]
            # Sort and re-add
            entries.sort(key=itemgetter(*cols))
            entries.insert(0, head)
            # Write
            self.write(entries)
        else:
            lines.sort(key=itemgetter(*cols))
            self.write(lines)

    def write(self, rows: List[List[str]]) -> None:
        """Writes contents of rows to self.path.

        Warning
        -------
            If the file already has content, that will be overwritten!
            This mirrors the modes used by `open()`_

        .. _open(): 
            https://docs.python.org/3.7/library/functions.html#open

        Parameters
        ----------
        rows : List[List[str]]
            A list of strings to write to self.path. `rows[0]` represents line 1,
            and `rows[0][0]` is line 1, column 1.

        Returns
        -------
        None

        Examples
        --------
        >>> example.write([['a', 'b', 'c']])

        """
        with open(self.path, "w", newline="") as file:
            writer = csv.writer(file, delimiter=self.delimiter)
            writer.writerows(rows)
        self.length = len(rows)

    def write_col(self, col: List[str], index: int = 0) -> None:
        """Writes contents of col to self.path at specified index

        Warning
        -------
            If the column already has content, that will be overwritten!
            This mirrors the modes used by `open()`_

        .. _open(): 
            https://docs.python.org/3.7/library/functions.html#open

        Parameters
        ----------
        col : List[str]
            A list of strings to write to self.path. This should be the same length as
            `self.length`
        index : int
            Which column to write at. Remember, Python is 0-indexed.

        Returns
        -------
        None

        Raises
        ------
        IndexError
            When `col` has more or less items than `self.length`
        
        Examples
        --------
        >>> example.write_col(['d'], index=2)

        """
        if len(col) != self.length:
            raise IndexError(
                f"col does not have the right number of items! n_lines = {self.length}, n_items = {len(col)}."
            )
        lines = self.read()
        write_col = itemsetter(index)
        for line, val in zip(lines, col):
            write_col(line, val)
        self.write(lines)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'todonepy'}

packages = \
['commands', 'helpers', 'subcommands']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['to = __init__:to']}

setup_kwargs = {
    'name': 'todonepy',
    'version': '4.0.8',
    'description': 'A developed command-line task manager',
    'long_description': "ToDonePY - A basic command-line tast manager\n============================================\n\n.. image:: https://www.repostatus.org/badges/latest/active.svg\n   :alt: Project Status: Active – The project has reached a stable, usable state and is being actively developed.\n   :target: https://www.repostatus.org/#active\n\n.. image:: https://img.shields.io/badge/License-GPLv3-blue.svg\n   :target: https://www.gnu.org/licenses/gpl-3.0\n   :alt: GPLv3 License\n   \n.. image:: https://img.shields.io/pypi/v/todonepy\n   :target: https://pypi.org/project/todonepy\n   :alt: PyPI\n\n.. image:: https://img.shields.io/pypi/pyversions/todonepy\n   :target: https://pypi.org/project/todonepy\n   :alt: PyPI - Python Versions\n\n.. image:: https://travis-ci.org/rbpatt2019/ToDonePy.svg?branch=master\n   :target: https://travis-ci.org/rbpatt2019/ToDonePy\n   :alt: Build Status\n   \n.. image:: https://readthedocs.org/projects/todonepy/badge/?version=latest\n   :target: https://todonepy.readthedocs.io/en/latest/?badge=latest\n   :alt: Documentation Status\n   \n.. image:: https://codecov.io/gh/rbpatt2019/ToDonePy/branch/master/graph/badge.svg\n   :target: https://codecov.io/gh/rbpatt2019/ToDonePy\n   :alt: Code Coverage\n\n.. image:: https://pyup.io/repos/github/rbpatt2019/ToDonePy/shield.svg\n     :target: https://pyup.io/repos/github/rbpatt2019/ToDonePy/\n     :alt: Updates\n\n.. image:: https://img.shields.io/badge/code%20style-black-000000.svg\n   :target: https://github.com/ambv/black\n   :alt: Codestyle: Black\n\nIntroduction\n------------\n\nMove your ToDo's to ToDone's!\n\n`ToDonePy <https://github.com/rbpatt2019/ToDonePy/>`_ is a command-line interface for managing your to do list. It provides a root command, `to`_, and three subcommands:\n\n- `to do`_ adds a new task to your list at different priorities.\n- `to doing`_ shows you what you should be doing.\n- `to done`_ removes a completed tast from your list.\n\nDocs and Code\n-------------\n\nThe documentation lives at https://ToDonePy.readthedocs.io/ .\n\nThe code lives at https://github.com/rbpatt2019/ToDonePy/ .\n\nInstallation\n------------\n\nThis project has been released on `PyPI <https://pypi.org>`_, so it can be installed with `pip`:\n\n.. code:: sh\n\n        pip install -U ToDonePy\n\nAlternatively, you can install the project manually by cloning the  `repo <https://github.com/rbpatt2019/ToDonePy>`_, and using the included Makefile.\n\n.. code:: sh\n\n    git clone https://github.com/rbpatt2019/ToDonePy/\n    make install\n    \nIf you would like to contribute to development, the install instructions are slightly different. Please see the section on `contributing <https://todonepy.readthedocs.io/en/latest/contributing.html#contributing>`_.\n\nUsage\n-----\n\n.. _to:\n\nThe base command `to`\n~~~~~~~~~~~~~~~~~~~~~\n\nThe base command `to` has a few useful features of its own. To see what version of the command you are using, call:\n\n.. code:: sh\n\n        to --version\n\nAs with any good command-line tool, you can get some basic help by calling:\n\n.. code:: sh\n\n        to --help\n\nYou can get help on any subcommand by calling `--help` after that subcommand. For example, to get help with `to doing`, call:\n\n.. code:: sh\n\n        to doing --help\n\nUnder the hood, `to` creates a Filer object that holds the information on the file you use for tracking you're TODOs. If you don't specify a file to use, it will default to `$HOME/.TODO.tsv`. If you would like to specify a different file to use, than call the command with the `--file/-f` flag like so:\n\n.. code:: sh\n        \n        to --file /path/to/your/TODO.tsv subcommand\n\n.. note:: If you plan to use a file other than the default, I recommend setting it by creating the environmental variable, `TODO_LIST`. \n\nRegardless of whether you use the default or not, calling `to` with any of the subcommands - `do`, `doing`, or `done` - will check to see if the file exists. If it does exist, `to` then pass the path on to the subcommand. If it doesn't exist, then `to` creates an empty file which it then passes on to the subcommand.\n\nAs a final note, it is worth emphasising that the contex object is only created when `to` is invoked with a subcommand. So, after a clean install, calling `to --help` or `to --version` will NOT create your `TODO.tsv` file, even if you pass the `--file/-f` flag. However, call `to do`, and it will pop into existence.\n\n.. _to do:\n\nAdding new tasks with `to do`\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\nTo begin tracking your TODOs, call the command as follows:\n\n.. code:: sh\n\n        to do rank tasks\n\n`to` is the base command. It must be invoked to use any part of the tool. The `do` subcommand is how you add tasks to your `TODO.tsv`. After `to do`, there are two mandatory arguments: `rank` and `tasks`. The first argument is `rank`. `rank` should be a number indicating how important this task is. 1 is very important, 2 less so, etc. Though nothing explicitly bans you from using as many ranks as you want, I would reccomed using 3 for high, medium, and low priority. \n\nThe second argument is `tasks`. Here, specify what it is you need to do. If your task takes more than one word to describe, then you need to include it in quotes. `tasks` supports an indefinite number of arguments, from 1 to as many as you want. \n\n.. note:: All tasks specified will be added at the same rank, so only combine tasks you want to give the same priority.\n\nSo, if you wanted to remind yourself to write an abstract for that paper you have been delaying and to email your boss, call:\n\n.. code:: sh\n        \n        to do 1 'Write my abstract' 'Email boss'\n\nThis will create `TODO.tsv` if it does not already exist, and add 'Write my abstract' and 'Email boss', both with a rank of one, to `TODO.tsv`. `to do` also logs the date and time the task was added, so that you always know how old a task is.\n\nSometimes, you want to sort your tasks as you add them. You can do that with the `--sort/-s` option. This specifies how to sort your list after a new task is added. It must be one of: `[rank, date, both, none]`. `both` sorts by rank and then date, and `none` does not sort, simply appending tasks to the end of your list. It defaults to `none`, on the grounds its better not to do something unless you ask. `Explicit is better than implicit <https://www.python.org/dev/peps/pep-0020/>`_, as they say. If you just wanted to sort by date after adding a new task, then you could call:\n\n.. code:: sh\n\n        to --sort do date 1 'Important work'\n\n.. note:: `--sort` follows the root command `to` as it directly impacts the file and is an option accessible to all subcommands.\n\n\n.. _to doing:\n\nKeeping track of tasks with `to doing`\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\nOnce you have added some TODOs to your list, you need to make sure you stay on top of them. To see what needs to be done, call:\n\n.. code:: sh\n\n        to doing\n\nThis should echo the 5 tasks at the top of your `TODO.tsv` to the terminal.\n\nYou can specify how to sort your tasks by passing the `--sort/-s` flag with one of: `[rank, date, both, none]`. It defaults to `none`, thus preserving the order in your `TODO.tsv`. Any call to sort will also change the order currently in your `TODO.tsv`, not just the order they are echoed.\n\nAlso, specifying the `--number/-n` flag will let you change how many tasks are returned, and it defaults to 5. So, if you want to return 3 tasks sorted by rank, call:\n\n.. code:: sh\n        \n        to -s rank doing -n 3\n\n.. note:: Remember, `-s` is a root command option!\n\nMaybe you prefer a graphic reminder instead of echoing in the terminal - I find this useful for spawning reminders while I am coding in VIM. `ToDonePy` has that covered, too! Just call:\n\n.. code:: sh\n        \n        to doing --reminder\n\nto trigger a notification window. By default, it stays up for 5 seconds. Currently, you can not set the time, though that's in the works!\n\n.. Note:: The graphic flag makes a system call to `notify-send`. If you don't have that installed, the command will fail. It should be installed on most Linux systems, though.\n\nSometimes, you might want to correct an error, change a priority, or in some way edit yout `TODO.tsv`. In these cases, you can call `to doing` in editor mode:\n\n.. code:: sh\n\n        to doing --edit\n        \nThis will open `TODO.tsv` in your system editor. Where you would see something like below, if you have been following along:\n\n.. code:: sh\n\n        ID      Rank    Date    Task\n        1       1       YYYY-MM-DD HH:MM     Write my abstract\n        2       1       YYYY-MM-DD HH:MM     Email boss\n        3       1       YYYY-MM-DD HH:MM     Important work\n\nNothing fancy, just a plain tsv with `ID` in the first column, `rank` in the second column, the date/time of addition in the third, and `task` in the fourth. Now, you can make all the changes you want, then save and close the file to return to the command line.\n\nCalling `--edit` will trump any calls to `sort` or `number` made in the same command.      \n\nThis call opens the default editor on your system, usually defined by the environmental variable EDITOR for Linux systems. If this variable is undefined, then it defaults to VIM - which should be your choice anyways! :P If that command is not found, then it will thros an OSError.\n\n\n.. _to done:\n\nCompleting your tasks with `to done`\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\nAfter the end of a productive work session, you have completed a task from your list. Boom! Time well spent. To remove it from your `TODO.tsv`, call:\n\n.. code:: sh\n\n        to done tasks\n\nAs with `to do`_, `to done` suports an indefinite number of tasks, as long as all multi-word tasks are enclosed in quotes. For example, if you emailed your boss that finished abstract, then you can remove those tasks like so:\n\n.. code:: sh\n        \n        to done 'Write my abstract' 'Email boss'\n\nIf `to done` finds these tasks in your `TODO.tsv`, it'll remove them! If it can't find the tasks, it will print a message saying which ones couldn't be removed.\n\nUnder the hood, `to done` creates a temp file, then performs a string match to each line of your `TODO.tsv`. If a perfect match to ''task'' is not in a line, that line is written to the temp file. If ''task'' is in a line, that line is skipped. This way, the temp file ends up containing only those tasks that aren't completed. Once every line is checked, the temp file replaces `TODO.tsv` with its contents. Task deleted!\n\n.. Warning:: If two different tasks contain the same text, they will both be deleted!\n\nKnown Bugs\n----------\n- Test hang when testing \n\nRecent Changes\n--------------\n\nPlease see the `CHANGELOG <https://github.com/rbpatt2019/ToDonePy/blob/master/CHANGELOG.rst>`_\n\nNext Steps\n----------\n\n- Addition of TODOs from file parsing\n- Support removal of tasks by task ID number\n- Full, OS-independent graphic interface\n\n",
    'author': 'rbpatt2019',
    'author_email': 'rb.patterson.cross@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://pypi.org/project/ToDonePy/',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



"""The `done` subcommand for the `to` main command"""
import argparse


def done(args: argparse.Namespace) -> None:
    """Remove a task to your list

    This command uses the supplied tasks to look for matches in your TODO
    list. A helpful message lets you know if the task(s) was(were) found and deleted.
    
    Note
    ----
        If your task contains more than one word, then each task must be enclosed in
        quotes. Otherwise, the CLI treats each word as a task. Also note that if
        multiple lines match a task, they will ALL be deleted.

    Parameters
    ----------
    args : argparse.Namespace
        Arguments forwarded from the CLI. For this subcommand, this includes:
    args.file : Filer
        The TODO file to be searched. From the root `to` command
    args.task : List[str]
        The list of tasks to be deleted
 
    Returns
    -------
    None
        Though a message will be echoed letting you know if the task(s) was(were)
        deleted successfully.

    Example
    -------
    $ to done 'An example' 'Is always helpful'

    """
    for item in args.tasks:
        if args.file.delete(item):
            ids = [str(x) for x in range(1, args.file.length)]
            ids.insert(0, "ID")
            args.file.write_col(ids, 0)
            print(f'Task "{item}" successfully deleted!')
        else:
            print(f'Task "{item}" not in TODOs...')



from typing import Any, Callable, List


def itemsetter(*items: int) -> Callable[[List, Any], None]:
    """Return a callable object that sets item from its operand
    
    This is essentially the opposite of `operator.itemgetter`_.
    If only one position is specified, the resulting callable will set that item.
    If multiple positions are specified, it sets all items

    .. _operator.itemgetter:
        https://docs.python.org/3.7/library/operator.html

    Parameters
    ----------
    *items : int
        The indices to be set. Remember, Python is 0-indexed

    Returns
    -------
    Callable[[List, Any], None]
        A function that will set the indices specified in `items` to a given value.

    Examples
    --------
    >>> x = ['a', 'b', 'c']
    >>> f = itemsetter(2)
    >>> f(x, 'z')
    >>> print(x)
    ['a', 'b', 'z']

    """

    def g(obj: List, *values: Any) -> None:
        for item, value in zip(items, values):
            obj[item] = value

    return g



# -*- coding: utf-8 -*-
"""The root `to` command

The root command provides several options.

`-s/--sort` allows you to specify how to sort added or returned tasks. Bear in mind
that this sorts the underlying file! It defaults to 'none' - best not to do anything
unless you need to!

`-f/file` allows you to specify where to store your TODOs. If you don't specify, it
defaults to `~/.todo.tsv` and will create the file if it doesn't exists. If you want
to keep your file elsewhere, you can specify that with the env var TODO_FILE

Like all good CLIs, `-v/--version` returns the version while `-h/--help` help for the
root command. Help for the subcommands can be found by calling `-h` after a subcommand,
like this: `to do -h`.

Attributes
----------
__version__ : str
    The version number, pulled from the pyproject.toml file
__todo__ : Filer
    The Filer object containing the TODOs. It first checks to see if the env var
    TODO_FILE is set. If it is, it looks there. If not, it defaults to ~/.todo.tsv.
    A hidden file is used to prevent clutter.
"""
import argparse
import os
from pathlib import Path

import poetry_version
from helpers.filer import Filer
from subcommands.do import do
from subcommands.doing import doing
from subcommands.done import done

__version__ = poetry_version.extract(source_file=__file__)
__todo__ = Filer(os.environ.get("TODO_FILE", Path.home() / ".todo.tsv"), create=True)

# The root command
to_parser = argparse.ArgumentParser(prog="to")
to_parser.add_argument(
    "-v",
    "--version",
    action="version",
    version=f"ToDonePy v{__version__}",
    help="Display the version and exit",
)
to_parser.add_argument(
    "-f", "--file", default=__todo__, type=Filer, help="Your TODO file",
)
to_parser.add_argument(
    "-s",
    "--sort",
    type=str,
    choices=["both", "none", "rank", "date"],
    default="none",
    help="How to sort TODOs",
)
subparsers = to_parser.add_subparsers(title="Sub-Commands")

# The do sub-command
do_parser = subparsers.add_parser("do", help="Add a new task")
do_parser.add_argument("rank", type=int, help="Importance of added tasks")
do_parser.add_argument(
    "tasks", type=str, nargs="*", help="Tasks to add. Each task should be in quotes"
)
do_parser.set_defaults(func=do)

# The doing sub-command
doing_parser = subparsers.add_parser("doing", help="See your current tasks")
doing_parser.add_argument(
    "-n", "--number", type=int, default=5, help="How many tasks to return"
)
doing_parser.add_argument(
    "-e",
    "--edit",
    nargs="?",
    type=bool,
    default=False,
    const=True,
    help="Launch TODO file in editor",
)
doing_parser.add_argument(
    "-r",
    "--reminder",
    nargs="?",
    type=bool,
    default=False,
    const=True,
    help="Whether to use a pop-up reminder",
)
doing_parser.set_defaults(func=doing)

# The done Sub-command
done_parser = subparsers.add_parser("done", help="Remove a completed task")
done_parser.add_argument(
    "tasks",
    type=str,
    nargs="*",
    help="The tasks to be deleted. Each task should be in quotes",
)
done_parser.set_defaults(func=done)



import shutil
import subprocess
from typing import List


def external_command(args: List[str]) -> subprocess.CompletedProcess:
    """Make a generic command line call

    Any command line call can be made. Pass the respective components as individual
    strings. Roughly speaking, anywhere there is a space, break it into a new
    component. See the documentation on `subprocess.run`_ for advanced use cases.

    .. _subprocess.run:
        https://docs.python.org/3.7/library/subprocess.html#subprocess.Popen

    Note
    ----
        If run in a situation where the user was providing a dynamic input, there are
        obvious security risks. In the app, however, the user cannot provide their own
        input, which I believe sufficiently mitigates the risk in this use case. 
        Obviously, if you adopt and use this function elsewhere, take care to check
        your inputs!

    Parameters
    ----------
    *args : List[str]
        The parts of the external command

    Returns
    -------
    subprocess.CompletedProcess
        If successful. This contains a number of useful attributes, including
        returncode and stdout.

    Raises
    ------
    OSError
        If unsuccessful. This will be thrown if the command found in args[0]
        cannot be found on the OS 
    subprocess.CalledProcessError
        If the called command returns a non-zero exit status
        
    Examples
    --------
    The results of a successful command ccan be queried like so:

    >>> results = external_command(['echo', 'hello']) 
    >>> results.returncode
    0

    """

    if shutil.which(args[0]) is None:
        raise OSError(f"Command {args[0]} not found!")
    return subprocess.run(list(args), check=True, capture_output=True)
